import logging
import json
import zmq
import os

from util import *

logger = logging.getLogger(__name__)


class TabooJudger:
    def __init__(self, task_setting, attacker_host="127.0.0.1", attacker_port=10086, defender_host="127.0.0.1",
                 defender_port=10087):
        self.task_setting = task_setting
        self.history = {
            "task_setting": task_setting
        }

        self.attacker_context = zmq.Context()
        self.attacker = self.attacker_context.socket(zmq.REQ)
        self.attacker.connect("tcp://%s:%d" % (attacker_host, attacker_port))

        self.attacker.send_json({
            "code": INIT,
            "data": self.task_setting
        })
        data = self.attacker.recv_json()
        if data["code"] != ATTACKER_FEEDBACK:
            logger.error("Unknown code, bad attacker.")
            raise NotImplementedError
        else:
            self.attacker_name = data["data"]

        logger.info("Connected to attacker %s:%d" % (attacker_host, attacker_port))

        self.defender_context = zmq.Context()
        self.defender = self.defender_context.socket(zmq.REQ)
        self.defender.connect("tcp://%s:%d" % (defender_host, defender_port))

        self.defender.send_json({
            "code": INIT,
            "data": self.task_setting
        })
        data = self.defender.recv_json()
        if data["code"] != DEFENDER_FEEDBACK:
            logger.error("Unknown code, bad defender.")
            raise NotImplementedError
        else:
            self.defender_name = data["data"]

        logger.info("Connected to defender %s:%d" % (defender_host, defender_port))

    def select_word(self, word_list):
        self.history["word_list"] = word_list
        logger.info("Word candidate set: %s" % json.dumps(word_list, ensure_ascii=False))
        self.attacker.send_json({
            "code": WORD_SELECT,
            "data": word_list
        })
        data = self.attacker.recv_json()
        if data["code"] != WORD_SELECT:
            logger.error("Unknown code, bad attacker.")
            raise NotImplementedError

        self.word = word_list[data["data"]]
        self.history["select_word"] = self.word
        logger.info("Attacker selects the word: %s" % self.word)

        self.attacker.send_json({
            "code": RECEIVED,
            "data": RECEIVED
        })

    def play_game(self, checker, formatter, ruler):
        self.history["sentences"] = []

        while True:
            attack_msg = self.attacker.recv_json()
            if attack_msg["code"] != ATTACK:
                logger.error("Unknown code, bad attacker.")
                raise NotImplementedError

            attack_msg = formatter.format(attack_msg["data"])

            self.history["sentences"].append(attack_msg)
            logger.info("Attacker says: %s" % attack_msg)

            if not ruler.check_available(attack_msg) or not ruler.check_relevance(attack_msg,
                                                                                  self.history["sentences"]):
                logger.error("Bad quality of attacker")
                self.history["result"] = "Bad quality of attacker."
                self.attacker.send_json({
                    "code": BAD_ATTACKER,
                    "data": BAD_ATTACKER
                })
                self.defender.send_json({
                    "code": BAD_ATTACKER,
                    "data": BAD_ATTACKER
                })
                break

            self.defender.send_json({
                "code": DEFEND,
                "data": attack_msg
            })

            defend_msg = self.defender.recv_json()
            if defend_msg["code"] == ACTIVELY_GUESS:
                guess_list = self.history["word_list"]
                self.history["guess_word_list"] = guess_list
                self.defender.send_json({
                    "code": GUESS_LIST,
                    "data": self.history["word_list"]
                })
                data = self.defender.recv_json()
                if data["code"] != GUESS:
                    logger.error("Unknown code, bad defender.")
                    raise NotImplementedError
                self.history["guess_id"] = data["data"]
                if guess_list[data["data"]] == self.history["word"]:
                    self.attacker.send_json({
                        "code": DEFENDER_WIN,
                        "data": guess_list[data["data"]]
                    })
                    self.defender.send_json({
                        "code": DEFENDER_WIN,
                        "data": DEFENDER_WIN
                    })
                    self.history["result"] = "Defender wins."
                    logger.info("Defender wins")
                    break
                else:
                    self.attacker.send_json({
                        "code": ATTACKER_WIN,
                        "data": {
                            "guess_list": guess_list,
                            "idx": data["data"]
                        }
                    })
                    self.defender.send_json({
                        "code": ATTACKER_WIN,
                        "data": self.word
                    })
                    self.history["result"] = "Attacker wins."
                    logger.info("Attacker wins")
                    break
            elif defend_msg["code"] == DEFEND:
                defend_msg = formatter.format(defend_msg["data"])
                self.history["sentences"].append(defend_msg)
                logger.info("Defender says: %s" % defend_msg)

                if not ruler.check_available(defend_msg) or not ruler.check_relevance(defend_msg,
                                                                                      self.history["sentences"]):
                    logger.error("Bad quality of defender")
                    self.history["result"] = "Bad quality of defender."
                    self.attacker.send_json({
                        "code": BAD_DEFENDER,
                        "data": BAD_DEFENDER
                    })
                    self.defender.send_json({
                        "code": BAD_DEFENDER,
                        "data": BAD_DEFENDER
                    })
                    break

                if checker.check(defend_msg, self.word):
                    self.attacker.send_json({
                        "code": ATTACKER_WIN,
                        "data": defend_msg
                    })
                    self.defender.send_json({
                        "code": ATTACKER_WIN,
                        "data": self.word
                    })
                    self.history["result"] = "Attacker wins."
                    logger.info("Attacker wins")
                    break
                else:
                    self.attacker.send_json({
                        "code": ATTACK,
                        "data": defend_msg
                    })
            else:
                logger.error("Unknown code, bad defender.")
                raise NotImplementedError


if __name__ == "__main__":
    os.system("clear")
    logging.basicConfig(format='%(asctime)s - %(levelname)s - %(name)s -   %(message)s',
                        datefmt='%m/%d/%Y %H:%M:%S',
                        level=logging.INFO)

    judger = TabooJudger({}, attacker_port=23333)
    judger.select_word(["apple", "peach", "ubuntu", "windows", "hello", "what", "fuck"])

    from taboo import formatter, ruler, checker

    judger.play_game(checker.TabooEnglishChecker(), formatter.TabooStupidFormatter(), ruler.TabooStupidRuler())



import logging
import zmq
import os

from util import *

logger = logging.getLogger(__name__)


class TabooDefender:
    def __init__(self, host="127.0.0.1", port=10087, name="Defender"):
        self.name = name

        self.context = zmq.Context()
        self.connection = self.context.socket(zmq.REP)
        self.connection.bind("tcp://%s:%d" % (host, port))

        data = self.connection.recv_json()

        if data["code"] == INIT:
            logger.info("Connection to server established.")
            self.task_setting = data["data"]
            self.connection.send_json({
                "code": DEFENDER_FEEDBACK,
                "data": self.name
            })
        else:
            logger.error("Unknown codes from server, raise error")
            raise NotImplementedError

    def get_task_setting(self):
        return self.task_setting

    def receive_msg(self):
        data = self.connection.recv_json()
        if data["code"] == ATTACKER_CORRUPT:
            logger.info("Attacker corrupts")
        logging.info("Attacker says: %s" % data["data"])
        if data["code"] in END_CODE_SET:
            if data["code"] == ATTACKER_WIN:
                logger.info("Attacker wins")
            elif data["code"] == DEFENDER_WIN:
                logger.info("Defender wins")
            elif data["code"] == DRAW:
                logger.info("Draw")

        return data

    def defend(self, sent):
        self.connection.send_json({
            "code": DEFEND,
            "data": sent
        })

    def begin_guessing(self):
        self.connection.send_json({
            "code": ACTIVELY_GUESS,
            "data": ACTIVELY_GUESS
        })

        data = self.connection.recv_json()

        return data["data"]

    def guess_word(self, p):
        self.connection.send_json({
            "code": GUESS,
            "data": p
        })

        if data["code"] in END_CODE_SET:
            if data["code"] == ATTACKER_WIN:
                logger.info("Attacker wins")
            elif data["code"] == DEFENDER_WIN:
                logger.info("Defender wins")
            elif data["code"] == DRAW:
                logger.info("Draw")

        return data


if __name__ == "__main__":
    os.system("clear")
    logging.basicConfig(format='%(asctime)s - %(levelname)s - %(name)s -   %(message)s',
                        datefmt='%m/%d/%Y %H:%M:%S',
                        level=logging.INFO)

    defender = TabooDefender()

    while True:
        data = defender.receive_msg()
        if data["code"] in END_CODE_SET:
            break
        print("Type your sentence: ", end='')
        sent = input().strip()

        defender.defend(sent)



import logging
import zmq
import os

from util import *

logger = logging.getLogger(__name__)


class TabooAttacker:
    def __init__(self, host="127.0.0.1", port=10086, name="Attacker"):
        self.name = name

        self.context = zmq.Context()
        self.connection = self.context.socket(zmq.REP)
        self.connection.bind("tcp://%s:%d" % (host, port))

        data = self.connection.recv_json()

        if data["code"] == INIT:
            logger.info("Connection to server established.")
            self.task_setting = data["data"]
            self.connection.send_json({
                "code": ATTACKER_FEEDBACK,
                "data": self.name
            })
        else:
            logger.error("Unknown codes from server, raise error")
            raise NotImplementedError

        data = self.connection.recv_json()

        if data["code"] == WORD_SELECT:
            logger.info("Receive word list from judger.")
            self.word_list = data["data"]
        else:
            logger.error("Unknown codes from server, raise error")
            raise NotImplementedError

    def get_task_setting(self):
        return self.task_setting

    def select_word(self, idx):
        logger.info("Selects the word: %s" % self.word_list[idx])
        self.connection.send_json({
            "code": WORD_SELECT,
            "data": idx
        })

        self.connection.recv_json()

    def attack(self, sent):
        self.connection.send_json({
            "code": ATTACK,
            "data": sent
        })
        data = self.connection.recv_json()
        if data["code"] == DEFENDER_CORRUPT:
            logger.info("Defender corrupts")
        logging.info("Defender says: %s" % data["data"])
        if data["code"] in END_CODE_SET:
            if data["code"] == ATTACKER_WIN:
                logger.info("Attacker wins")
            elif data["code"] == DEFENDER_WIN:
                logger.info("Defender wins")
            elif data["code"] == DRAW:
                logger.info("Draw")

        return data


if __name__ == "__main__":
    os.system("clear")
    logging.basicConfig(format='%(asctime)s - %(levelname)s - %(name)s -   %(message)s',
                        datefmt='%m/%d/%Y %H:%M:%S',
                        level=logging.INFO)

    attacker = TabooAttacker(port=23333)
    attacker.select_word(0)

    while True:
        print("Type your sentence: ", end='')
        sent = input().strip()
        data = attacker.attack(sent)

        if data["code"] in END_CODE_SET:
            break



MAX_BUFFER = 65536

# Codes for initialization
INIT = 0
ATTACKER_FEEDBACK = 1
DEFENDER_FEEDBACK = 2

# Codes for word selection
WORD_SELECT = 10
RECEIVED = 11

# Codes for playing game
ATTACK = 20
DEFEND = 21
ACTIVELY_GUESS = 22
BEGIN_GUESS = 23
GUESS_LIST = 24
GUESS = 25

# Codes for game ends
GAME_ENDS = 30
ATTACKER_WIN = 31
DEFENDER_WIN = 32
DRAW = 33
ATTACKER_CORRUPT = 34
DEFENDER_CORRUPT = 35
BAD_ATTACKER = 36
BAD_DEFENDER = 37

END_CODE_SET = {GAME_ENDS, ATTACKER_WIN, DEFENDER_WIN, DRAW, ATTACKER_CORRUPT, DEFENDER_CORRUPT, BAD_ATTACKER,
                BAD_DEFENDER}



import nltk
import jieba


class TabooEnglishChecker:
    def __init__(self):
        pass

    def check(self, sent, word):
        segmentation = nltk.word_tokenize(sent)
        if word in segmentation:
            return True
        else:
            return False


class TabooChineseCheck:
    def __init__(self):
        pass

    def check(self, sent, word):
        segmentation = list(jieba.cut(sent))
        if word in segmentation:
            return True
        else:
            return False
